@@ -24,12 +24,15 @@ class MessageLogger(object):
     def __init__(self, message_func):
         self.send_message = message_func
 
-    def _log_data(self, level, message):
-        self.send_message("log", level, message)
+    def _log_data(self, action, **kwargs):
+        self.send_message("log", action, kwargs)
+
+    def process_output(self, process, data, command):
+        self._log_data("process_output", process=process, data=data, command=command)
 
 def _log_func(level_name):
     def log(self, message):
-        self._log_data(level_name.lower(), message)
+        self._log_data(level_name.lower(), message=message)
     log.__doc__ = """Log a message with level %s
 
 :param message: The string message to log
@@ -123,8 +126,12 @@ class TestRunner(object):
 
 
 def start_runner(test_queue, runner_command_queue, runner_result_queue,
-                 browser, executor_cls, executor_kwargs, stop_flag):
+                 executor_cls, executor_kwargs,
+                 executor_browser_cls, executor_browser_kwargs,
+                 stop_flag):
+    """Launch a TestRunner in a new process"""
     try:
+        browser = executor_browser_cls(**executor_browser_kwargs)
         executor = executor_cls(browser, **executor_kwargs)
         with TestRunner(test_queue, runner_command_queue, runner_result_queue, executor) as runner:
             try:
@@ -133,6 +140,7 @@ def start_runner(test_queue, runner_command_queue, runner_result_queue,
                 stop_flag.set()
     except Exception as e:
         runner_result_queue.put(("log", ("critical", traceback.format_exc())))
+        print >> sys.stderr, traceback.format_exc()
         stop_flag.set()
     finally:
         runner_command_queue = None
@@ -276,7 +284,7 @@ class TestRunnerManager(threading.Thread):
         return self.child_stop_flag.is_set() or self.parent_stop_flag.is_set()
 
     def init(self):
-        """Create the Firefox process that is being tested,
+        """Launch the browser that is being tested,
         and the TestRunner process that will run the tests."""
         #It seems that this lock is helpful to prevent some race that otherwise
         #sometimes stops the spawned processes initalising correctly, and
@@ -293,13 +301,13 @@ class TestRunnerManager(threading.Thread):
             if self.command_queue:
                 self.command_queue.put(("init_failed", ()))
             else:
+                self.logger.debug("Setting child stop flag in init_failed")
                 self.child_stop_flag.set()
 
 
         with self.init_lock:
             #To guard against cases where we fail to connect with marionette for
             #whatever reason
-            #TODO: make this timeout configurable
             self.init_timer = threading.Timer(self.browser.init_timeout, init_failed)
             try:
                 self.init_timer.start()
@@ -334,14 +342,17 @@ class TestRunnerManager(threading.Thread):
     def start_test_runner(self):
         assert self.command_queue is not None
         assert self.remote_queue is not None
+        executor_browser_cls, executor_browser_kwargs = self.browser.executor_browser()
+        args=(self.tests_queue,
+              self.remote_queue,
+              self.command_queue,
+              self.executor_cls,
+              self.executor_kwargs,
+              executor_browser_cls,
+              executor_browser_kwargs,
+              self.child_stop_flag)
         self.test_runner_proc = Process(target=start_runner,
-                                        args=(self.tests_queue,
-                                              self.remote_queue,
-                                              self.command_queue,
-                                              self.browser,
-                                              self.executor_cls,
-                                              self.executor_kwargs,
-                                              self.child_stop_flag),
+                                        args=args,
                                         name="Thread-TestRunner-%i" % self.manager_number)
         self.test_runner_proc.start()
         self.logger.debug("Test runner started")
@@ -471,8 +482,8 @@ class TestRunnerManager(threading.Thread):
         self.stop_runner()
         self.init()
 
-    def log(self, level, message):
-        getattr(self.logger, level)(message)
+    def log(self, action, kwargs):
+        getattr(self.logger, action)(**kwargs)
 
     def error(self, message):
         self.logger.error(message)
