@@ -80,11 +80,11 @@ def setup_stdlib_logger():
 
 
 def do_test_relative_imports(test_root):
-    global serve
+    global serve, manifest
 
     sys.path.insert(0, os.path.join(test_root))
     sys.path.insert(0, os.path.join(test_root, "tools", "scripts"))
-    import serve
+    import serve, manifest
 
 
 class TestEnvironmentError(Exception):
@@ -161,10 +161,9 @@ class TestEnvironment(object):
 
     def restore_files(self):
         for path in self.files_to_restore:
+            os.unlink(path)
             if os.path.exists(path + ".orig"):
                 os.rename(path + ".orig", path)
-            else:
-                os.unlink(path)
 
 class TestChunker(object):
     def __init__(self, total_chunks, chunk_number):
@@ -315,12 +314,21 @@ class TestLoader(object):
         self.metadata_root = metadata_root
         self.test_filter = test_filter
         self.run_info = run_info
+        self.manifest_path = os.path.join(self.metadata_root, "MANIFEST.json")
         self.manifest = self.load_manifest()
         self.tests = None
 
+    def create_manifest(self):
+        logger.info("Creating test manifest")
+        manifest.setup_git(self.test_path)
+        manifest_file = manifest.Manifest(None)
+        manifest.update(manifest_file)
+        manifest.write(manifest_file, self.manifest_path)
+
     def load_manifest(self):
-        metadata.do_test_relative_imports(self.tests_root)
-        return metadata.manifest.load(os.path.join(self.metadata_root, "MANIFEST.json"))
+        if not os.path.exists(self.manifest_path):
+            self.create_manifest()
+        return manifest.load(self.manifest_path)
 
     def get_test(self, manifest_test, expected_file):
         if expected_file is not None:
@@ -332,6 +340,25 @@ class TestLoader(object):
     def load_expected_manifest(self, test_path):
         return manifestexpected.get_manifest(self.metadata_root, test_path, self.run_info)
 
+    def iter_tests(self, test_types, chunker=None):
+        manifest_items = self.test_filter(self.manifest.itertypes(*test_types))
+
+        for test_path, tests in manifest_items:
+            expected_file = self.load_expected_manifest(test_path)
+            for manifest_test in tests:
+                test = self.get_test(manifest_test, expected_file)
+                test_type = manifest_test.item_type
+                yield test_path, test_type, test
+
+    def get_disabled(self, test_types):
+        rv = defaultdict(list)
+
+        for test_path, test_type, test in self.iter_tests(test_types):
+            if test.disabled():
+                rv[test_type].append(test)
+
+        return rv
+
     def load_tests(self, test_types, chunk_type, total_chunks, chunk_number):
         """Read in the tests from the manifest file and add them to a queue"""
         rv = defaultdict(list)
@@ -343,13 +370,9 @@ class TestLoader(object):
                    "equal_time": EqualTimeChunker}[chunk_type](total_chunks,
                                                                chunk_number)
 
-        for test_path, tests in chunker(manifest_items):
-            expected_file = self.load_expected_manifest(test_path)
-            for manifest_test in tests:
-                test = self.get_test(manifest_test, expected_file)
-                test_type = manifest_test.item_type
-                if not test.disabled():
-                    rv[test_type].append(test)
+        for test_path, test_type, test in self.iter_tests(test_types, chunker):
+            if not test.disabled():
+                rv[test_type].append(test)
 
         return rv
 
@@ -430,6 +453,16 @@ def list_test_groups(tests_root, metadata_root, test_types, product, **kwargs):
     for item in sorted(test_loader.get_groups(test_types)):
         print item
 
+def list_disabled(tests_root, metadata_root, test_types, product, **kwargs):
+    rv = []
+    run_info = wpttest.get_run_info(product, debug=False)
+    test_loader = TestLoader(tests_root, metadata_root, TestFilter(), run_info)
+
+    for test_type, tests in test_loader.get_disabled(test_types).iteritems():
+        for test in tests:
+            rv.append({"test": test.id, "reason": test.disabled()})
+    print json.dumps(rv, indent=2)
+
 def run_tests(tests_root, metadata_root, product, **kwargs):
     logging_queue = None
     logging_thread = None
@@ -489,6 +522,7 @@ def run_tests(tests_root, metadata_root, product, **kwargs):
                 unexpected_count = 0
                 logger.suite_start(test_ids, run_info)
                 for test_type in kwargs["test_types"]:
+                    logger.info("Running %s tests" % test_type)
                     tests_queue = test_queues[test_type]
 
                     executor_cls = executor_classes.get(test_type)
@@ -564,5 +598,7 @@ def main():
 
     if args.list_test_groups:
         list_test_groups(**kwargs)
+    elif args.list_disabled:
+        list_disabled(**kwargs)
     else:
         return run_tests(**kwargs)
